childProcess = require "child_process"
fs = require "fs"
path = require "path"
util = require "util"
mkdirp = require "mkdirp"
_ = require "underscore"

##### Shell commands
#
# Run shell commands

# Run a command with streamed output
module.exports.executeCommand = executeCommand = (command, options, callback) ->
  if typeof options is "undefined"
    options = {}
  else if typeof options is "function"
    callback = options
    options = {}
    
  _(options).defaults
    output: off
    
  parts = command.split " "
  command = parts.shift()
  args = parts
  
  spawnedProcess = childProcess.spawn command, args
  
  ###
  spawnedProcess.stdin.pipe process.stdin,
    end: off
  spawnedProcess.stdout.pipe process.stdout,
    end: off
  spawnedProcess.stderr.pipe process.stderr,
    end: off
  ###
  
  spawnedProcess.stdout.setEncoding "utf8"
  spawnedProcess.stdout.on "data", (data) ->
    if options.output is yes then process.stdout.write data
    callback? null, data, null
    
  spawnedProcess.stderr.setEncoding "utf8"
  spawnedProcess.stderr.on "data", (data) ->
    if options.output is yes then process.stdout.write data
    callback? data, null, null
    
  spawnedProcess.on "exit", (code) ->
    callback? null, null, code
  
# Run a command with buffered output
module.exports.runCommand = runCommand = (command, options, callback) ->
  if typeof options is "undefined"
    options = {}
  else if typeof options is "function"
    callback = options
    options = {}
    
  _(options).defaults
    output: off
    
  spawnedProcess = childProcess.exec command, encoding: "utf8", (error, stdout, stderr) ->
    if stdout?
      if options.output is yes then process.stdout.write stdout
    if stderr?
      if options.output is yes then process.stdout.write stderr
    
    callback? error, stdout, stderr
    
# Run a file with buffered output
module.exports.runFile = runFile = (file, options, callback) ->
  if typeof options is "undefined"
    options = {}
  else if typeof options is "function"
    callback = options
    options = {}
    
  _(options).defaults
    output: off
    
  parts = file.split " "
  file = parts.shift()
  args = parts
  spawnedProcess = childProcess.execFile file, args, encoding: "utf8", (error, stdout, stderr) ->
    if stdout?
      if options.output is yes then process.stdout.write stdout
    if stderr?
      if options.output is yes then process.stdout.write stderr
    
    callback? error, stdout, stderr
      
##### File system
#
# Manipulate folders and files

# Generate a file from a template file
#
# - Create the folders if they don't exist
# - Overwrite the file if it exists
module.exports.generateFileFromFile = generateFileFromFile = (templatePath, filePath, locals, callback) ->
  fs.readFile templatePath, "utf8", (error, data) ->
    if error? then throw error
    
    generateFileFromString data, filePath, locals, callback
        
# Generate a file from a template string
#
# - Create the folders if they don't exist
# - Overwrite the file if it exists
module.exports.generateFileFromString = generateFileFromString = (data, filePath, locals, callback) ->
  data = ejs.render data,
    locals: locals
  
  dirPath = path.dirname filePath
  mkdirp dirPath, 0755, (error) ->
    if error? then throw error
    fs.writeFile filePath, data, "utf8", (error) ->
      if error? then throw error
      callback null
      
##### Logging

module.exports.log = log = (message) ->
  if typeof message is "object"
    console.log util.inspect message, yes, null
  else if typeof message is "string"
    console.log message
  
#### Conversion

module.exports.readFile = (path, callback) ->
  fs.readFile path, "utf8", (error, data) ->
    if error then throw error
    callback data
    
module.exports.readFileSync = (path) ->
  fs.readFileSync path, "utf8"
    
module.exports.jsonEncode = (object) ->
  JSON.stringify object, null, 2
  
module.exports.jsonDecode = (string) ->
  JSON.parse string