require "should"
fs = require "fs"
rimraf = require "rimraf"
mkdirp = require "mkdirp"
yiu = require "../lib/yobiutil-api"

describe "shell", ->
  
  describe "executeCommand", ->
    
    it "should execute command with streamed output", (done) ->
      yiu.executeCommand "echo pass", output: off, (errorData, stdoutData, exitCode) ->
        if errorData?
          errorData.should.equal null
        if stdoutData?
          stdoutData.should.equal "pass\n"
        if exitCode?
          exitCode.should.equal 0
          done()
          
    it "should execute command with no options", (done) ->
      yiu.executeCommand "echo pass", (errorData, stdoutData, exitCode) ->
        if errorData?
          errorData.should.equal null
        if stdoutData?
          stdoutData.should.equal "pass\n"
        if exitCode?
          exitCode.should.equal 0
          done()
          
  describe "runCommand", ->
    
    it "should run command with buffered output", (done) ->
      yiu.runCommand "echo pass", output: off, (error, stdout, stderr) ->
        if error? then return done error
        stderr.should.equal ""
        stdout.should.equal "pass\n"
        done()
        
    it "should run command with no options", (done) ->
      yiu.runCommand "echo pass", (error, stdout, stderr) ->
        if error? then return done error
        stderr.should.equal ""
        stdout.should.equal "pass\n"
        done()
        
  describe "runFile", ->
    
    it "should run file with buffered output", (done) ->
      yiu.runFile "/bin/echo pass", output: off, (error, stdout, stderr) ->
        if error? then return done error
        stderr.should.equal ""
        stdout.should.equal "pass\n"
        done()
        
    it "should run file with no options", (done) ->
      yiu.runFile "/bin/echo pass", (error, stdout, stderr) ->
        if error? then return done error
        stderr.should.equal ""
        stdout.should.equal "pass\n"
        done()
          
describe "fs", ->
  
  tempDir = __dirname + "/../tmp"
  destDir = "#{tempDir}/dest"
  destFile = "#{destDir}/file.md"
  
  afterEach (done) ->
    rimraf tempDir, done
    
  describe "generateFileFromFile()", ->
    
    srcDir = "#{tempDir}/src"
    srcFile = "#{srcDir}/file.md"
    
    beforeEach (done) ->
      mkdirp srcDir, 0755, (error) ->
        if error? then return done error
        fs.writeFile srcFile, "This was generated from a <%= what %>", "utf8", done
    
    it "should generate a file based on a template file", (done) ->
      
      yiu.generateFileFromFile srcFile, destFile, what: "file", (error) ->
        if error? then return done error
        fs.readFile destFile, "utf8", (error, data) ->
          if error? then return done error
          data.should.equal "This was generated from a file"
          done()
          
  describe "generateFileFromString()", ->
    
    srcString = "This was generated from a <%= what %>"
    
    it "should generate a file based on a template string", (done) ->
      
      yiu.generateFileFromString srcString, destFile, what: "string", (error) ->
        if error? then return done error
        fs.readFile destFile, "utf8", (error, data) ->
          if error? then return done error
          data.should.equal "This was generated from a string"
          done()